// ignore_for_file: library_private_types_in_public_api, no_logic_in_create_state

import 'dart:convert';

import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:google_fonts/google_fonts.dart';

import '../../flutterbudpay.dart';
import '../common/utils.dart';
import '../constant/constant.dart';
import '../constant/submitbutton.dart';
import '../models/checkout_response.dart';
import '../request/apis.dart';
import '../ui/checkout/base_checkout.dart';
import '../ui/checkout/checkout_widget.dart';
import '../ui/common/bouncing_dots.dart';
import '../ui/error_widget.dart';
import '../ui/sucessful_widget.dart';
import 'copy.dart';
import 'transactionstatus.dart';

class Banktransfer extends StatefulWidget {
  final Charge charge;
  final String secretkey, publickey;
  final OnResponse<CheckoutResponse> onResponse;
  const Banktransfer(
      {Key? key,
      required this.secretkey,
      required this.publickey,
      required this.charge,
      required this.onResponse})
      : super(key: key);

  @override
  _BanktransferState createState() => _BanktransferState(charge, onResponse);
}

class _BanktransferState extends BaseCheckoutMethodState<Banktransfer> {
  final Charge _charge;
  _BanktransferState(this._charge, OnResponse<CheckoutResponse> onResponse)
      : super(onResponse, CheckoutMethod.bank);

  String? _paymentError;
  bool _paymentSuccessful = false;
  String amount = '0';
  var gettingFee = false;
  @override
  void initState() {
    super.initState();
    getfee({"amount": ((_charge.amount)/100).toString(), "currency": _charge.currency},
        widget.secretkey);
  }

  getfee(body, key) async {
    String res;
    setState(() {
      gettingFee = true;
    });
    res = await getTxnFee(body, key);
    var cmddetails = jsonDecode(res);

    setState(() {
      gettingFee = false;
    });
    if (cmddetails['status']) {
      amount = cmddetails['data']['total_amount'].toString();
    }
  }

  CheckoutResponse? _response;
  void _onProcessingChange(bool processing) {
    setState(() {
      processing = processing;
    });
  }

  var isloading = false;
  void _onPaymentResponse(CheckoutResponse response) {
    if (kDebugMode) {
      print(response);
    }
    _response = response;
    if (!mounted) return;
    setState(() {
      isloading = false;
    });
    if (response.status == true) {
      _onPaymentSuccess();
    } else {
      _onPaymentError(response.message);
    }
  }

  void _onPaymentSuccess() {
    setState(() {
      _paymentSuccessful = true;
      _paymentError = null;
      _onProcessingChange(false);
    });
  }

  void _onPaymentError(String? value) {
    setState(() {
      _paymentError = value;
      _paymentSuccessful = false;
      _onProcessingChange(false);
    });
  }

  @override
  Widget buildAnimatedChild() {
    return isloading
        ? Transactionstatus(
            charge: _charge,
            secretkey: widget.secretkey,
            verify: true,
            // service: widget.bankService,
            onResponse: _onPaymentResponse,
          )
        : _showProcessingError()
            ? _buildErrorWidget()
            : _paymentSuccessful
                ? _buildSuccessfulWidget()
                : Material(
                    color: Colors.white,
                    elevation: 5.0,
                    borderRadius: BorderRadius.circular(5),
                    child: Padding(
                      padding: const EdgeInsets.all(20.0),
                      child: Column(
                        crossAxisAlignment: CrossAxisAlignment.start,
                        children: [
                          const SizedBox(
                            height: 30,
                          ),
                          Container(
                            decoration: const BoxDecoration(
                                borderRadius:
                                    BorderRadius.all(Radius.circular(5)),
                                color: primarycolour),
                            height: 65,
                            padding: const EdgeInsets.symmetric(horizontal: 20),
                            child: Row(
                              children: [
                                Text(
                                  "Amount",
                                  style: GoogleFonts.dmSans(
                                      color: Colors.white,
                                      fontSize: 16,
                                      fontWeight: FontWeight.bold),
                                ),
                                const Spacer(),
                                gettingFee
                              ? const BouncingDots()
                              : Text(
                                  Utils.formatcharge(amount, _charge.currency!),
                                  style: GoogleFonts.dmSans(
                                      color: Colors.white,
                                      fontSize: 24,
                                      fontWeight: FontWeight.bold),
                                ),
                              ],
                            ),
                          ),
                          const SizedBox(
                            height: 40,
                          ),
                          Text(
                            "Bank Transfer",
                            style: GoogleFonts.dmSans(
                                fontSize: 16, fontWeight: FontWeight.bold),
                          ),
                          const SizedBox(
                            height: 10,
                          ),
                          const Divider(),
                          const SizedBox(
                            height: 10,
                          ),
                          Row(
                            children: [
                              Text(
                                "Bank Name",
                                style: GoogleFonts.dmSans(
                                    fontSize: 14, fontWeight: FontWeight.w500),
                              ),
                              const Spacer(),
                              Text(
                                _charge.account!.bank!.name!,
                                style: GoogleFonts.dmSans(
                                    fontSize: 14, fontWeight: FontWeight.bold),
                              ),
                            ],
                          ),
                          const SizedBox(
                            height: 10,
                          ),
                          const Divider(),
                          const SizedBox(
                            height: 10,
                          ),
                          Row(
                            children: [
                              Text(
                                "Account Number",
                                style: GoogleFonts.dmSans(
                                    fontSize: 14, fontWeight: FontWeight.w500),
                              ),
                              const Spacer(),
                              Text(
                                _charge.account!.number!,
                                style: GoogleFonts.dmSans(
                                    fontSize: 14, fontWeight: FontWeight.bold),
                              ),
                              const SizedBox(
                                width: 20,
                              ),
                              Copy(
                                quote: _charge.account!.number!,
                              )
                            ],
                          ),
                          const SizedBox(
                            height: 10,
                          ),
                          const Divider(),
                          const SizedBox(
                            height: 10,
                          ),
                          Row(
                            children: [
                              Text(
                                "Account Name",
                                style: GoogleFonts.dmSans(
                                    fontSize: 14, fontWeight: FontWeight.w500),
                              ),
                              const Spacer(),
                              Text(
                                _charge.account!.name!,
                                style: GoogleFonts.dmSans(
                                    fontSize: 14, fontWeight: FontWeight.bold),
                              ),
                            ],
                          ),
                          const SizedBox(
                            height: 10,
                          ),
                          const Divider(),
                          const SizedBox(
                            height: 40,
                          ),
                          // const Spacer(),
                          Submitbutton(
                            name: "I have sent the money",
                            press: () {
                              setState(() {
                                isloading = true;
                              });
                            },
                          ),
                          const SizedBox(
                            height: 20,
                          ),
                          Row(
                            mainAxisAlignment: MainAxisAlignment.center,
                            children: [
                              const Icon(Icons.lock, size: 12),
                              Text(
                                'Secured By',
                                style: GoogleFonts.dmSans(
                                    fontWeight: FontWeight.bold, fontSize: 12),
                              ),
                              const Text(''),
                              Image.asset(
                                'assets/images/logo.png',
                                width: 24,
                                height: 24,
                                package: 'flutterbudpay',
                              ),
                              Text(
                                'BudPay',
                                style: GoogleFonts.dmSans(
                                  fontWeight: FontWeight.bold,
                                  color: primarycolour,
                                  fontSize: 12,
                                ),
                              ),
                            ],
                          ),
                          const SizedBox(
                            height: 20,
                          ),
                        ],
                      ),
                    ),
                  );
  }

  _showProcessingError() {
    return !(_paymentError == null || _paymentError!.isEmpty);
  }

  Widget _buildErrorWidget() {
    return ErrorPage(
      text: _paymentError,
      method: CheckoutMethod.bank,
      isCardPayment: _charge.card!.isValid(),
      vSync: this,
      payWithBank: () {
        setState(() {
          _onPaymentError(null);
          _charge.card = PaymentCard.empty();
          _paymentError = null;
        });
      },
      tryAnotherCard: () {
        setState(() {
          _onPaymentError(null);
          _charge.card = PaymentCard.empty();
        });
      },
      startOverWithCard: () {
        _onPaymentError(null);
      },
    );
  }

  Widget _buildSuccessfulWidget() => SuccessfulWidget(
        amount: _charge.amount,
        onCountdownComplete: () {
          if (_response!.card != null) {
            _response!.card!.nullifyNumber();
          }
          Navigator.of(context).pop(_response);
        },
      );
}
