// ignore_for_file: library_private_types_in_public_api, no_logic_in_create_state, use_build_context_synchronously

import 'dart:convert';

import 'package:flutter/material.dart';
import 'package:flutterbudpay/src/ui/common/bouncing_dots.dart';
import 'package:flutterbudpay/src/wallet/transactionstatus.dart';
import 'package:google_fonts/google_fonts.dart';

import '../../flutterbudpay.dart';
import '../common/utils.dart';
import '../constant/constant.dart';
import '../constant/submitbutton.dart';
import '../models/bank.dart';
import '../models/checkout_response.dart';
import '../request/apis.dart';
import '../ui/checkout/base_checkout.dart';
import '../ui/checkout/checkout_widget.dart';

class Fundwallet extends StatefulWidget {
  final Charge charge;
  final OnResponse<CheckoutResponse> onResponse;
  final String secretKey;
  const Fundwallet(
      {Key? key,
      required this.charge,
      required this.onResponse,
      required this.secretKey})
      : super(key: key);

  @override
  _FundwalletState createState() => _FundwalletState(charge, onResponse);
}

class _FundwalletState extends BaseCheckoutMethodState<Fundwallet> {
  int selected = 1;
  var isLoading = false;
  var gettingFee = false;
  var gettingPaymentMethods = false;
  String amount = '0';
  String excluded = '';
  String ref = "refpay${DateTime.now().microsecondsSinceEpoch}";
  final Charge _charge;
  _FundwalletState(this._charge, OnResponse<CheckoutResponse> onResponse)
      : super(onResponse, CheckoutMethod.bank);
  @override
  void initState() {
    super.initState();
    getfee({
      "amount": ((_charge.amount) / 100).toString(),
      "currency": _charge.currency
    }, widget.secretKey);
    paymentMethods(widget.secretKey);
  }

  getfee(body, key) async {
    String res;
    setState(() {
      gettingFee = true;
    });
    res = await getTxnFee(body, key);
    var cmddetails = jsonDecode(res);

    setState(() {
      gettingFee = false;
    });
    if (cmddetails['status']) {
      amount = cmddetails['data']['total_amount'].toString();
    }
  }

  paymentMethods(key) async {
    String res;
    setState(() {
      gettingPaymentMethods = true;
    });
    res = await getPaymentMethods(key);
    var cmddetails = jsonDecode(res);

    setState(() {
      gettingPaymentMethods = false;
    });
    if (cmddetails['status']) {
      excluded = cmddetails['data'].toString();
    }
  }

  void initiate() async {
    String res;
    var jsonBody = {
      "email": _charge.email,
      "amount": (_charge.amount ~/ 100).toString(),
      "currency": _charge.currency,
      "reference": _charge.reference
    };
    setState(() {
      isloading = true;
    });
    res = await deposittransfer(jsonBody, widget.secretKey);

    var cmddetails = jsonDecode(res);

    isloading = false;
    if (cmddetails['status']) {
      var response = CheckoutResponse(
          message: cmddetails['message'],
          reference: _charge.reference,
          status: false,
          card: _charge.card?..nullifyNumber(),
          method: CheckoutMethod.bank,
          verify: false);
      _charge.account ??= BankAccount(
          Bank(cmddetails['data']['bank_name'], 1),
          cmddetails['data']['account_number'],
          cmddetails['data']['account_name']);
      onResponse(response);
    } else {
      var response = CheckoutResponse(
          message: cmddetails['message'],
          reference: _charge.reference,
          status: false,
          card: _charge.card?..nullifyNumber(),
          method: CheckoutMethod.bank,
          verify: false);
      Navigator.pop(context, response);
    }
    setState(() {});
  }

  var isloading = false;
  @override
  Widget buildAnimatedChild() {
    return isloading
        ? Transactionstatus(
            charge: _charge,
            secretkey: widget.secretKey,
            verify: false,
            // service: widget.bankService,
            onResponse: _onPaymentResponse,
          )
        : Center(
            child: Material(
              color: Colors.white,
              elevation: 5.0,
              borderRadius: BorderRadius.circular(5),
              child: gettingPaymentMethods
                  ? const SizedBox(
                      height: 400,
                      width: 300,
                      child: BouncingDots(color: Color(0xff6C30F4)),
                    )
                  : Column(
                      crossAxisAlignment: CrossAxisAlignment.start,
                      children: [
                        // Padding(
                        //   padding: const EdgeInsets.fromLTRB(15,10,0,0),
                        //   child: Image.asset(
                        //     'assets/images/logo.png',
                        //     height: 60,
                        //     width: 60,
                        //   ),
                        // ),
                        const SizedBox(
                          height: 30,
                        ),
                        Padding(
                          padding: const EdgeInsets.fromLTRB(20, 20, 20, 0),
                          child: Container(
                            decoration: const BoxDecoration(
                                borderRadius:
                                    BorderRadius.all(Radius.circular(5)),
                                color: primarycolour),
                            height: 65,
                            padding: const EdgeInsets.symmetric(horizontal: 20),
                            child: Row(
                              children: [
                                Text(
                                  "Amount",
                                  style: GoogleFonts.dmSans(
                                      color: Colors.white,
                                      fontSize: 16,
                                      fontWeight: FontWeight.bold),
                                ),
                                const Spacer(),
                                gettingFee
                                    ? const BouncingDots()
                                    : Text(
                                        Utils.formatcharge(
                                            amount, _charge.currency!),
                                        style: GoogleFonts.dmSans(
                                            color: Colors.white,
                                            fontSize: 24,
                                            fontWeight: FontWeight.bold),
                                      ),
                              ],
                            ),
                          ),
                        ),
                        const SizedBox(
                          height: 40,
                        ),
                        Padding(
                          padding: const EdgeInsets.fromLTRB(20, 0, 20, 0),
                          child: Text(
                            "Transfer Method",
                            style:
                                GoogleFonts.dmSans(fontWeight: FontWeight.bold),
                          ),
                        ),
                        const SizedBox(height: 10),
                        const Divider(height: 1),
                        excluded.contains('transfer')
                            ? Container()
                            : Container(
                                color: selected == 1
                                    ? Colors.grey.withOpacity(0.1)
                                    : Colors.white,
                                child: Column(children: [
                                  const SizedBox(height: 20),
                                  Padding(
                                    padding:
                                        const EdgeInsets.fromLTRB(20, 0, 20, 0),
                                    child: GestureDetector(
                                      child: Row(
                                        children: [
                                          Container(
                                            height: 20,
                                            width: 20,
                                            decoration: BoxDecoration(
                                                borderRadius:
                                                    const BorderRadius.all(
                                                        Radius.circular(30)),
                                                border: Border.all(
                                                    color: selected == 1
                                                        ? primarycolour
                                                        : const Color(
                                                            0xffD3D3D3),
                                                    width: 5)),
                                          ),
                                          const SizedBox(
                                            width: 20,
                                          ),
                                          Text(
                                            "Bank Transfer",
                                            style: GoogleFonts.dmSans(),
                                          )
                                        ],
                                      ),
                                      onTap: () {
                                        setState(() {
                                          selected = 1;
                                        });
                                      },
                                    ),
                                  ),
                                  const SizedBox(height: 20),
                                ]),
                              ),
                        const Divider(height: 1),
                        excluded.contains('card')
                            ? Container()
                            : Container(
                                color: selected == 2
                                    ? Colors.grey.withOpacity(0.1)
                                    : Colors.white,
                                child: Column(
                                  children: [
                                    const SizedBox(height: 20),
                                    Padding(
                                      padding: const EdgeInsets.fromLTRB(
                                          20, 0, 20, 0),
                                      child: GestureDetector(
                                        child: Row(
                                          children: [
                                            Container(
                                              height: 20,
                                              width: 20,
                                              decoration: BoxDecoration(
                                                  borderRadius:
                                                      const BorderRadius.all(
                                                          Radius.circular(30)),
                                                  border: Border.all(
                                                      color: selected == 2
                                                          ? primarycolour
                                                          : const Color(
                                                              0xffD3D3D3),
                                                      width: 5)),
                                            ),
                                            const SizedBox(
                                              width: 20,
                                            ),
                                            Text(
                                              "Debit Card",
                                              style: GoogleFonts.dmSans(),
                                            )
                                          ],
                                        ),
                                        onTap: () {
                                          setState(() {
                                            selected = 2;
                                          });
                                        },
                                      ),
                                    ),
                                    const SizedBox(height: 20),
                                  ],
                                ),
                              ),
                        const Divider(height: 1),
                        const SizedBox(height: 15),
                        Padding(
                          padding: const EdgeInsets.fromLTRB(20, 0, 20, 0),
                          child: Submitbutton(
                            isloading: isLoading,
                            name: "Continue",
                            press: () async {
                              if (selected == 1) {
                                initiate();
                                // onResponse(CheckoutResponse(
                                //     loading: true,
                                //     message: "loading",
                                //     reference: _charge.reference,
                                //     status: false,
                                //     card: _charge.card?..nullifyNumber(),
                                //     method: CheckoutMethod.bank,
                                //     verify: false));
                              } else {
                                onResponse(CheckoutResponse(
                                    message: "loading",
                                    reference: _charge.reference,
                                    status: false,
                                    card: _charge.card?..nullifyNumber(),
                                    method: CheckoutMethod.card,
                                    verify: false));
                              }
                            },
                          ),
                        ),
                        const SizedBox(
                          height: 10,
                        ),
                        Padding(
                          padding: const EdgeInsets.fromLTRB(0, 0, 0, 20),
                          child: Row(
                            mainAxisAlignment: MainAxisAlignment.center,
                            children: [
                              const Icon(Icons.lock, size: 12),
                              Text(
                                'Secured By',
                                style: GoogleFonts.dmSans(
                                    fontWeight: FontWeight.bold, fontSize: 12),
                              ),
                              const Text(''),
                              Image.asset(
                                'assets/images/logo.png',
                                width: 24,
                                height: 24,
                                package: 'flutterbudpay',
                              ),
                              Text(
                                'BudPay',
                                style: GoogleFonts.dmSans(
                                  fontWeight: FontWeight.bold,
                                  color: primarycolour,
                                  fontSize: 12,
                                ),
                              ),
                            ],
                          ),
                        )
                      ],
                    ),
            ),
          );
  }

  void _onPaymentResponse(CheckoutResponse response) {
    // print(response);
    // _response = response;
    // if (!mounted) return;
    // setState(() {
    //   isloading = false;
    // });
    // if (response.status == true) {
    //   _onPaymentSuccess();
    // } else {
    //   _onPaymentError(response.message);
    // }
    //
    // if (response.loading && response.method == CheckoutMethod.bank) {
    //   initiate();
    // }
  }
}
